@@ -2,7 +2,7 @@
 //
 // AoIP stream routing configuration
 //
-// (C) 2014 Fred Gleason <fredg@paravelsystems.com>
+// (C) 2014-2018 Fred Gleason <fredg@paravelsystems.com>
 //
 //    This program is free software; you can redistribute it and/or modify
 //    it under the terms of version 2.1 of the GNU Lesser General Public
@@ -274,43 +274,24 @@ int SyRouting::dstMeterLevel(int slot,int chan) const
 }
 
 
-bool SyRouting::gpiState(int gpi,int line) const
-{
-  int slot=GetSlotByGpio(gpi);
-  bool ret=false;
-
-  if(slot>=0) {
-    ret=sy_gpi_states[slot+line];
-  }
-  return ret;
-}
-
-
 bool SyRouting::gpiStateBySlot(int slot,int line) const
 {
   return sy_gpi_states[slot*SWITCHYARD_GPIO_BUNDLE_SIZE+line];
 }
 
-
-void SyRouting::setGpi(int gpi,int line,bool state,bool pulse)
+void SyRouting::setGpi(int srcnum,int line,bool state,bool pulse)
 {
-  int slot=GetSlotByGpio(gpi);
-
-  if(slot>=0) {
-    sy_gpi_states[slot+line]=state;
+  for(unsigned i=0;i<gpis();i++) {
+    if(gpoAddress(i)==SyRouting::streamAddress(SyRouting::Stereo,srcnum)) {
+      sy_gpi_states[i+line]=state;
+    }
   }
 }
 
 
-bool SyRouting::gpoState(int gpo,int line) const
+void SyRouting::setGpiBySlot(int slot,int line,bool state,bool pulse)
 {
-  int slot=GetSlotByGpio(gpo);
-  bool ret=false;
-
-  if(slot>=0) {
-    ret=sy_gpo_states[slot+line];
-  }
-  return ret;
+  sy_gpi_states[slot+line]=state;
 }
 
 
@@ -320,16 +301,76 @@ bool SyRouting::gpoStateBySlot(int slot,int line) const
 }
 
 
-void SyRouting::setGpo(int gpo,int line,bool state,bool pulse)
+void SyRouting::setGpo(int srcnum,int line,bool state,bool pulse)
 {
-  int slot=GetSlotByGpio(gpo);
-
-  if(slot>=0) {
-    sy_gpo_states[slot+line]=state;
+  for(unsigned i=0;i<gpos();i++) {
+    if(gpoAddress(i)==SyRouting::streamAddress(SyRouting::Stereo,srcnum)) {
+      sy_gpo_states[i+line]=state;
+    }
   }
 }
 
 
+void SyRouting::setGpoBySlot(int slot,int line,bool state,bool pulse)
+{
+  sy_gpo_states[slot+line]=state;
+}
+
+
+SyRouting::GpoMode SyRouting::gpoMode(int slot) const
+{
+  return sy_gpo_modes[slot];
+}
+
+
+void SyRouting::setGpoMode(int slot,SyRouting::GpoMode mode)
+{
+  sy_gpo_modes[slot]=mode;
+}
+
+
+QHostAddress SyRouting::gpoAddress(int slot) const
+{
+  return QHostAddress(gpo_addr[slot]);
+}
+
+
+void SyRouting::setGpoAddress(int slot,const QHostAddress &addr)
+{
+  gpo_addr[slot]=addr.toIPv4Address();
+}
+
+
+void SyRouting::setGpoAddress(int slot,const QString &addr)
+{
+  gpo_addr[slot]=QHostAddress(addr).toIPv4Address();
+}
+
+
+int SyRouting::gpoSnakeSlot(int slot) const
+{
+  return sy_gpo_snake_slots[slot];
+}
+
+
+void SyRouting::setGpoSnakeSlot(int slot,int snake_slot)
+{
+  sy_gpo_snake_slots[slot]=snake_slot;
+}
+
+
+QString SyRouting::gpoName(int slot) const
+{
+  return sy_gpo_names[slot];
+}
+
+
+void SyRouting::setGpoName(int slot,const QString &str)
+{
+  sy_gpo_names[slot]=str;
+}
+
+
 unsigned SyRouting::nicQuantity() const
 {
   return sy_nic_devices.size();
@@ -440,6 +481,16 @@ void SyRouting::load()
     setDstAddress(i,s->value(key+"/DestinationAddress").toString());
     setDstName(i,s->value(key+"/DestinationName").toString());
   }
+  for(unsigned i=0;i<gpos();i++) {
+    QString key=QString().sprintf("Slot%u",i+1);
+    setGpoMode(i,(SyRouting::GpoMode)s->value(key+"/GpoMode").toInt());
+    setGpoAddress(i,s->value(key+"/GpoAddress").toString());
+    setGpoSnakeSlot(i,s->value(key+"/GpoSnakeSlot").toInt());
+    setGpoName(i,s->value(key+"/GpoName").toString());
+    if(gpoName(i).isEmpty()) {
+      setGpoName(i,QString().sprintf("GPIO %d",i+1));
+    }
+  }
   delete s;
 #else
   QString section;
@@ -462,6 +513,16 @@ void SyRouting::load()
     setDstAddress(i,p->addressValue(section,"DestinationAddress",""));
     setDstName(i,p->stringValue(section,"DestinationName",QString().sprintf("Destination %u",i+1)));
   }
+  for(unsigned i=0;i<gpos();i++) {
+    section=QString().sprintf("Slot%u",i+1);
+    setGpoMode(i,(SyRouting::GpoMode)p->intValue(section,"GpoMode"));
+    setGpoAddress(i,p->addressValue(section,"GpoAddress",""));
+    setGpoSnakeSlot(i,p->intValue(section,"GpoSnakeSlot"));
+    setGpoName(i,p->stringValue(section,"GpoName",gpoName(i)));
+    if(gpoName(i).isEmpty()) {
+      setGpoName(i,QString().sprintf("GPIO %d",i+1));
+    }
+  }
   delete p;
 #endif  // WIN32
 }
@@ -485,6 +546,12 @@ void SyRouting::save() const
       s->setValue(key+"/DestinationAddress",dstAddress(i).toString());
       s->setValue(key+"/DestinationName",dstName(i));
     }
+    if(i<(int)gpos()) {
+      s->setValue(key+"/GpoMode",(int)gpoMode(i));
+      s->setValue(key+"/GpoAddress",gpoAddress(i).toString());
+      s->setValue(key+"/GpoSnakeSlot",gpoSnakeSlot(i));
+      s->setValue(key+"/GpoName",gpoName(i));
+    }
   }
   delete s;
 #else
@@ -513,6 +580,13 @@ void SyRouting::save() const
 	      (const char *)dstAddress(i).toString().toAscii());
       fprintf(f,"DestinationName=%s\n",(const char *)dstName(i).toAscii());
     }
+    if(i<(int)gpos()) {
+      fprintf(f,"GpoMode=%d\n",gpoMode(i));
+      fprintf(f,"GpoAddress=%s\n",
+	      (const char *)gpoAddress(i).toString().toAscii());
+      fprintf(f,"GpoSnakeSlot=%d\n",gpoSnakeSlot(i));
+      fprintf(f,"GpoName=%s\n",(const char *)gpoName(i).toAscii());
+    }
     fprintf(f,"\n");
   }
   fclose(f);
@@ -541,15 +615,15 @@ QHostAddress SyRouting::streamAddress(SyRouting::Realm realm,uint16_t lw_num)
   QHostAddress ret;
   switch(realm) {
   case SyRouting::Stereo:    // 239.192.0.0/15
-    ret.setAddress(4022337536+lw_num);
+    ret.setAddress(4022337536u+lw_num);
     break;
 
   case SyRouting::Backfeed:  // 239.193.0.0/15
-    ret.setAddress(4022403072+lw_num);
+    ret.setAddress(4022403072u+lw_num);
     break;
 
   case SyRouting::Surround:  // 239.128.0.0/15
-    ret.setAddress(4022599680+lw_num);
+    ret.setAddress(4022599680u+lw_num);
     break;
   }
 
@@ -607,18 +681,6 @@ QString SyRouting::socketErrorString(const QString &msg)
 }
 
 
-int SyRouting::GetSlotByGpio(int gpio) const
-{
-  for(unsigned i=0;i<dstSlots();i++) {
-    if(SyRouting::livewireNumber(dstAddress(i))==(unsigned)gpio) {
-      return i*SWITCHYARD_GPIO_BUNDLE_SIZE;
-    }
-  }
-
-  return -1;
-}
-
-
 void SyRouting::LoadInterfaces()
 {
   if((sy_subscription_socket=socket(PF_INET,SOCK_DGRAM,IPPROTO_IP))<0) {
